#! python3
# -*- encoding: utf-8 -*-

import pytest
from dbgrab import get_engine_manager, check_data, DataExtractor

env_file= "cmft_conf/.env.cmft"
manager = get_engine_manager(env_file)

def test_extractor_mysql():
    credible_engine = manager.get_engine("CREDIBLE")

    # noinspection SqlNoDataSourceInspection
    sql_query = r"select * from credible_db.fetcher_task_record ftr limit 50"

    # 检查数据量
    sql_query_count = sql_query.lower().replace('select * from', "select count(*) from")
    max_info = check_data(credible_engine, sql_query_count)
    print("Count总数: ", max_info)

    # 抽取数据
    dte = DataExtractor(credible_engine, manager._config.WORK_PATH)
    dte.extractor.set_file("credible_test").to_csv(sql_query, chunk_size=20000)

def test_extractor_oceanbase():
    trade_engine = manager.get_engine("TRADE")

    # noinspection SqlNoDataSourceInspection
    sql_query = "SELECT * FROM TRADEDBADM.WTAORDLT t where ROWNUM <= 1000"

    # 检查数据量
    sql_query_count = sql_query.lower().replace('select * from', "select count(*) from")
    max_info = check_data(trade_engine, sql_query_count)
    print("Count总数: ", max_info)

    # 抽取数据
    dte = DataExtractor(trade_engine, manager._config.WORK_PATH)
    dte.extractor.set_file("trade_test").to_csv(sql_query, chunk_size=20000)


if __name__ == "__main__":
    pytest.main()



#! python3
# -*- encoding: utf-8 -*-

from typing import Optional
from pydantic import Field
from pydantic_settings import BaseSettings


class OceanbaseConfig(BaseSettings):
    """ 单个oceanbase oracle模式数据库  """
    # jdbc:oceanbase://localhost:2883/OCEANBASE_DB?useUnicode=true&characterEncoding=UTF-8&rewriteBatchedStatements=true
    JDBC: str = Field(..., description="数据库连接jdbc URL")

    # username@tenantname#clustername
    USERNAME: str = Field(..., description="用户名@租户名#集群名")
    PASSWORD: str = Field(..., description="密码")

    # com.oceanbase.jdbc.Driver
    DRIVER: str = Field(..., description="OceanBase驱动程序名称")

    # /soft/oceanbase/oceanbase-client-2.4.9.jar
    CLIENT_JAR: str = Field(..., description="OceanBase客户端jar包")





#! python3
# -*- encoding: utf-8 -*-
from functools import wraps
from dbgrab.extractor.dt_factory import (
    SqlExtractor,
    PgSqlExtractor,
    OracleSqlExtractor
)


class DataExtractor(object):
    """
    数据提取器，基于数据库引擎实例对象，初始化数据提取器
    """

    def __init__(self, engine, fp):
        """
        Args:
            engine: 数据库引擎实例，如 SQLAlchemy 或 JayDeBeEngine 实例
            fp: 数据提取工作目录路径
        """
        self._engine = engine
        self.extractor = self._init_extractor(fp)

    def _init_extractor(self, fp):
        if "postgresql:" in str(self._engine):
            extractor = PgSqlExtractor(self._engine, fp)

        elif "oracle:" in str(self._engine):
            extractor = OracleSqlExtractor(self._engine, fp)

        elif "oceanbase:" in str(self._engine):
            conn = self._engine.connect()
            extractor = OracleSqlExtractor(conn, fp)

        else:
            extractor = SqlExtractor(self._engine, fp)
            setattr(extractor, "to_csv_enhance", extractor.to_csv)

        return extractor

    def fetch_db_iter(self, chunk_size):
        _extractor = self.extractor
        def decorate(func):
            @wraps(func)
            def wrapper(*args, **kwargs):
                files = []
                _extractor.count = 0
                for file_name, end_date, fetch_sql, count in func(*args, **kwargs):

                    db_args = [fetch_sql]
                    db_kwargs = {
                        "chunk_size": chunk_size,
                        "max_lines": count,
                    }
                    if "postgresql:" not in str(self._engine ):
                        db_kwargs.pop("max_lines")

                    _extractor.set_file(file_name).to_csv_enhance(*db_args, **db_kwargs)
                    _extractor.write_chk_file(count, end_date)
                    files.append((file_name + '.csv', file_name + '.chk'))
                return files, _extractor.count
            return wrapper
        return decorate



#! python3
# -*- encoding: utf-8 -*-

import os
from datetime import datetime, timedelta
import calendar
import pandas as pd

from tqdm import tqdm
from rtsf.p_common import FileSystemUtils
from rtsf.p_applog import AppLog


# 获取给定日期范围，每个月的起止时间
def get_month_start_end_dates(start_date, end_date, fmt="%Y%m%d"):
    start_date = datetime.strptime(start_date, "%Y%m%d")
    end_date = datetime.strptime(end_date, "%Y%m%d")

    month_start_dates = []
    current_year = start_date.year
    current_month = start_date.month

    while True:
        month_start_date = datetime(current_year, current_month, 1)
        month_end_date = datetime(current_year, current_month, day=calendar.monthrange(current_year, current_month)[1])

        if month_start_date > end_date:
            break

        month_start_dates.append((month_start_date.strftime(fmt), month_end_date.strftime(fmt)))

        if current_month == 12:
            current_month = 1
            current_year += 1
        else:
            current_month += 1

    return month_start_dates


# 获取给定日期范围，每个年的起止时间
def get_year_start_end_dates(start_date, end_date, fmt="%Y%m%d"):
    start_date = datetime.strptime(start_date, "%Y%m%d")
    end_date = datetime.strptime(end_date, "%Y%m%d")

    year_start_dates = []
    current_year = start_date.year

    while True:
        year_start_date = datetime(year=current_year, month=1, day=1)
        year_end_date = datetime(year=current_year, month=12, day=31)

        if year_start_date > end_date:
            break

        year_start_dates.append((year_start_date.strftime(fmt), year_end_date.strftime(fmt)))

        current_year += 1

    return year_start_dates


# 获取给定日期范围，输出范围内每天的日期
def get_day_dates(start_date, end_date, fmt="%Y%m%d"):
    # get_day_dates("20231001", "20231007")

    start_date = datetime.strptime(start_date, "%Y%m%d")
    end_date = datetime.strptime(end_date, "%Y%m%d")

    # 使用 timedelta 逐日打印日期
    dates = []
    current = start_date
    while current <= end_date:
        dates.append((current.strftime(fmt), current.strftime(fmt)))
        current += timedelta(days=1)

    return dates


# 查看数据量
def check_data(conn, sql_query):
    """
    # 全部黑名单的数据量
    conn = conf.ORACLE_ENGINE
    check_data(conn, "SELECT count(*) FROM SWORDADM.BLACK_INFO t")
    # 转账到和包订单表 2023年全年的交易量
    conn = conf.PG_ENGINE
    check_data(conn, "SELECT count(1) FROM dd.ods_WTAORDLT t where t.ord_dt >= '20230101' and t.ord_dt <='20231231'")
    """
    conn = conn.raw_connection()
    cursor = conn.cursor()
    result = cursor.execute(sql_query)
    # Max num:  cursor.fetchall()
    return cursor.fetchone()



log = AppLog()
logger = log.get_logger()


class SqlExtractor(object):

    def __init__(self, engine, work_path):
        """
        Args:
            engine: 数据库引擎，如 SQLAlchemy 或 JayDeBeEngine 实例
            work_path: 数据提取工作目录路径
        """
        self.engine = engine
        # print(engine)
        self._file_name = None
        self._chk_file = None
        self._work_path = work_path
        self.count = 0

    def set_file(self, file_name):
        """
        :param file_name: 不带文件后缀的文件名
        """
        file_path, file_name = os.path.split(file_name)
        if file_path:
            raise Exception("Need file name without: %s" % file_path)

        file_name, ext = os.path.splitext(file_name)
        if ext:
            raise Exception("Need file name without: %s" % ext)

        csv_file = "{}.csv".format(file_name)
        self._file_name = os.path.join(self._work_path, csv_file)

        chk_file = "{}.chk".format(file_name)
        self._chk_file = os.path.join(self._work_path, chk_file)

        # FileSystemUtils.mkdirs(os.path.join(self._work_path, "logs"))
        # log_file = "{}_{}.log".format(file_name, datetime.now().strftime('%Y%m%d'))
        # log.to_file(os.path.join(self._work_path, "logs", log_file))

        return self

    def to_csv(self, sql_query, chunk_size=1024):
        """ read_sql_query的逻辑是
        1. 先用execute执行sql语句，然后在判断是否有chunksize，没有就直接返回所有数据，有的话根据chunksize返回一个iterator。
        2. 所以,使用chunk_size，这不是一个真正的分批次读取，如果数据量大，还是会导致内存爆炸直至卡死
        :param sql_query:
        :param chunk_size:
        :return:
        """

        if self._file_name is None:
            # print("Warning: need to call method 'set_file'.")
            logger.warning("Warning: need to call method 'set_file'.")
            return

        # print("Please wait...")
        logger.info("Please wait...")

        FileSystemUtils.force_delete_file(self._file_name)

        count_num = 0
        # self.count = 0
        chunks = pd.read_sql_query(sql_query, con=self.engine, chunksize=chunk_size)
        is_header = True
        for chunk in chunks:
            count_num += len(chunk)
            start_time = datetime.now()
            chunk.to_csv(self._file_name, mode="a", index=False, header=is_header, float_format='{:f}'.format, encoding='utf-8')
            is_header = False
            elapsed_time = datetime.now() - start_time
            # print(f'chunksize={chunk_size}: {elapsed_time.total_seconds()} seconds')
        # print("Save to '%s' success(%d)" % (self._file_name, count_num))
        self.count += count_num
        logger.info("Save to '%s' success(%d)" % (self._file_name, count_num))
        logger.info(f"Extracted: {self.count}\n---")

    def write_chk_file(self, count, date):
        if self._file_name is None:
            print("Warning: need to call method 'set_file'.")
            return

        if not os.path.exists(self._file_name):
            print(f"Warning: {self._file_name} is not exist. Will create a empty file.")
            with open(self._file_name, 'w') as f:
                pass

        file_size = os.path.getsize(self._file_name)
        with open(self._chk_file, 'w') as f:
            data = "{}|{}|{}|{}\t".format(os.path.split(self._file_name)[1], str(file_size), str(count), str(date))
            f.write(data)

class PgSqlExtractor(SqlExtractor):

    def __init__(self, engine, work_path):
        super().__init__(engine, work_path)

    def to_csv_enhance(self, sql_query, chunk_size=10000, offset=0, max_lines=None):
        """        构造 limit 子句，实现大数据的分批读取。
        :param sql_query: sql语句 不带limit 和 offset
        :param chunk_size: 每次取的行数，默认1万
        :param offset: 起始位，默认0
        :param max_lines: 最大取数, 默认一直取数; 若设置了，则打印总进度条
        :return:
        """
        if self._file_name is None:
            # print("Warning: need to call method 'set_file'.")
            logger.warning("Warning: need to call method 'set_file'.")
            return

        # print("Please wait...")
        logger.info("Please wait...")

        count_num = 0
        is_header = True
        FileSystemUtils.force_delete_file(self._file_name)

        progress_bar = tqdm(total=max_lines, unit='rows') if max_lines else None
        while True:
            try:
                # 从数据库中获取数据
                # query = f"{sql_query} LIMIT {chunk_size}  offset {offset}"
                query = f"{sql_query} OFFSET {offset} ROWS FETCH FIRST {chunk_size} ROWS ONLY;"

                logger.debug(f"Reading SQL offset: {offset}")
                df = pd.read_sql(query, self.engine)
                if df.empty:
                    logger.debug("==== Read SQL Finish. ====")
                    break

                length_df = len(df)
                count_num += length_df
                _ = progress_bar.update(length_df) if progress_bar is not None else None
                logger.debug(f"Updating csv file: {count_num}/{max_lines}")
                df.to_csv(self._file_name, index=False, header=is_header, mode='a')
                is_header = False

                offset += chunk_size
                if (max_lines is not None) and (offset >= max_lines):
                    break
            except Exception as e:
                # print("Error occurred while reading from database.")
                logger.error("Error occurred while reading from database.")
                raise e
        # print("Save to '%s' success(%d)" % (self._file_name, count_num))
        self.count += count_num
        logger.info("Save to '%s' success(%d)" % (self._file_name, count_num))
        logger.info(f"Extracted: {self.count}\n---")


class OracleSqlExtractor(SqlExtractor):

    def __init__(self, engine, work_path):
        super().__init__(engine, work_path)
        self.to_csv_enhance = self.to_csv




#! python3
# -*- encoding: utf-8 -*-

import os
import pytest
from dbgrab.configs import  get_db_config

def test_conf():
    # 使用自定义.env文件
    env_file = os.path.join(os.path.dirname(__file__), '.env.example')

    # 打印读取到的.env配置
    print("📄 读取到的.env配置:")
    config = get_db_config(env_file)
    print(f"数据模型配置: {config.model_config}")
    print(f"所有配置: {[i for i in dir(config) if i.isupper()]}")

    print(f"  数据库配置:")
    for db_name, db_config in config.DATABASES.items():
        print(f"    🔹 {db_name}:")
        # 处理字典类型的配置
        print(f"      URL: {db_config.URL}")
        print(f"      连接池大小: {db_config.POOL_SIZE}")
        print(f"      最大溢出连接数: {db_config.MAX_OVERFLOW}")
        print(f"      连接回收时间: {db_config.POOL_RECYCLE}秒")
        print(f"      输出SQL日志: {db_config.ECHO}")

    print(f"  DATABASES: {config.DATABASES.keys()}")
    print(f"    DATABASES: {config.DATABASES}")
    print(f"  OCEANBASES: {config.OCEANBASES.keys()}")
    print(f"    OCEANBASES: {config.OCEANBASES}")


if __name__ == "__main__":
    pytest.main()


#! python3
# -*- encoding: utf-8 -*-
class DynamicDBError(Exception):
    """动态数据库包基础异常"""
    pass

class ConfigError(DynamicDBError):
    """配置相关异常"""
    pass

class EngineError(DynamicDBError):
    """引擎相关异常"""
    pass

class ConnectionError(DynamicDBError):
    """连接相关异常"""
    pass


#! python3
# -*- encoding: utf-8 -*-
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from typing import Dict, Optional, Any
from contextlib import contextmanager

from dbgrab.configs import get_db_config, clear_config_cache
from dbgrab.dt_exceptions import EngineError
from dbgrab.extractor.dt_factory import logger
from dbgrab.jaydebe.jaydebe_engine import JayDeBeEngine


class DatabaseEngineManager:
    """数据库引擎管理器"""

    def __init__(self, env_file: Optional[str] = None):
        self._env_file = env_file
        self._config = get_db_config(env_file)
        self._engines: Dict[str, Any] = {}
        self._sessionmakers: Dict[str, Any] = {}

        self._initialize_engines()

    def get_engine(self, db_name: str) -> Any:
        """获取数据库引擎"""
        if db_name not in self._engines:
            raise EngineError(f"数据库引擎 '{db_name}' 不存在")

        return self._engines[db_name]

    @contextmanager
    def get_session(self, db_name: Optional[str]):
        """获取数据库会话的上下文管理器"""
        session_maker = self._get_sessionmaker(db_name)
        session = session_maker()
        try:
            yield session
            session.commit()
        except Exception:
            session.rollback()
            raise
        finally:
            session.close()

    def list_engines(self) -> Dict[str, Dict[str, Any]]:
        """列出所有引擎的配置信息（隐藏敏感信息）"""
        engine_info = {}
        for name, engine in self._engines.items():
            if hasattr(engine, "dialect"):
                engine_info[name] = {
                    'url': str(engine.url).split('@')[0] + '@***',  # 隐藏密码
                    'pool_size': engine.pool.size(),
                    'echo': engine.echo,
                    'dialect': engine.dialect.name
                }
            else:
                engine_info[name] = {
                    'url': engine._jdbc_url,
                    'dialect': "oceanbase"
                }
        return engine_info

    def reload_engines(self, env_file: Optional[str] = None):
        """重新加载引擎配置"""
        logger.info("重新加载数据库引擎配置...")

        # 关闭现有引擎
        self.close_all_engines()

        # 清除缓存并重新加载配置
        clear_config_cache()
        self._env_file = env_file or self._env_file
        self._config = get_db_config(self._env_file)

        # 重新初始化引擎
        self._initialize_engines()

    def close_all_engines(self):
        """关闭所有数据库引擎"""
        for name, engine in self._engines.items():
            try:
                engine.dispose()
                logger.info(f"🔒 数据库引擎 '{name}' 已关闭")
            except Exception as e:
                logger.warning(f"⚠️ 关闭数据库引擎 '{name}' 时出错: {e}")

        self._engines.clear()
        self._sessionmakers.clear()

    def _initialize_engines(self):
        """初始化所有数据库引擎"""
        logger.info(f"开始初始化数据库引擎...")

        for db_name, db_config in self._config.DATABASES.items():
            try:
                # 创建引擎
                engine = create_engine(
                    db_config.URL,
                    pool_size=db_config.POOL_SIZE,
                    max_overflow=db_config.MAX_OVERFLOW,
                    pool_recycle=db_config.POOL_RECYCLE,
                    echo=db_config.ECHO,
                    future=True
                )

                self._engines[db_name] = engine
                self._sessionmakers[db_name] = sessionmaker(bind=engine)

                logger.info(f"数据库引擎 '{db_name}' 初始化成功")

            except Exception as e:
                logger.error(f"数据库引擎 '{db_name}' 初始化失败: {e}")
                raise EngineError(f"数据库 {db_name} 初始化失败: {e}")

        for db_name, db_config in self._config.OCEANBASES.items():
            try:
                # 创建引擎
                engine = JayDeBeEngine(
                    jdbc_driver_name=db_config.DRIVER,
                    jdbc_url=db_config.JDBC,
                    jdbc_user=db_config.USERNAME,
                    jdbc_password=db_config.PASSWORD,
                    jdbc_jar=db_config.CLIENT_JAR
                )

                self._engines[db_name] = engine
                self._sessionmakers[db_name] = sessionmaker(bind=engine)

                logger.info(f"数据库引擎 '{db_name}' 初始化成功")

            except Exception as e:
                logger.error(f"数据库引擎 '{db_name}' 初始化失败: {e}")
                raise EngineError(f"数据库 {db_name} 初始化失败: {e}")

    def _get_sessionmaker(self, db_name: str) -> Any:
        """获取会话工厂"""
        if db_name not in self._sessionmakers:
            raise EngineError(f"会话工厂 '{db_name}' 不存在")

        return self._sessionmakers[db_name]

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close_all_engines()


# 全局引擎管理器实例
_global_engine_manager: Optional[DatabaseEngineManager] = None


def get_engine_manager(env_file: Optional[str] = None) -> DatabaseEngineManager:
    """获取全局引擎管理器"""
    global _global_engine_manager
    if _global_engine_manager is None:
        _global_engine_manager = DatabaseEngineManager(env_file)
    return _global_engine_manager


def set_global_engine_manager(env_file: Optional[str] = None):
    """设置全局引擎管理器"""
    global _global_engine_manager
    _global_engine_manager = DatabaseEngineManager(env_file)


#! python3
# -*- encoding: utf-8 -*-

from typing import Optional
from pydantic import Field
from pydantic_settings import BaseSettings


class DatabaseConfig(BaseSettings):
    """单个数据库配置模型"""
    NAME: str = Field(..., description="数据库别名/标识名称")
    URL: str = Field(..., description="数据库连接URL")
    DRIVER: Optional[str] = Field(default=None, description="数据库驱动")
    POOL_SIZE: int = Field(default=5, description="连接池大小")
    MAX_OVERFLOW: int = Field(default=10, description="最大溢出连接数")
    POOL_RECYCLE: int = Field(default=3600, description="连接回收时间(秒)")
    ECHO: bool = Field(default=False, description="是否输出SQL日志")




#! python3
# -*- encoding: utf-8 -*-
import re
from datetime import datetime, timedelta
from typing import Optional
from dbgrab import (
    check_data,
    logger,
    get_month_start_end_dates,
    get_year_start_end_dates,
    get_day_dates,
    DataExtractor,
)
from dbgrab.cons import JayDeBeConfig


class DataBaseFetcher(object):
    """
    数据库数据抓取器，基于数据库提取器和SQL模板文件抓取数据，并保存到csv
    """
    def __init__(self, env_file: str = ".env", sql_file: str = "sql.yml", chunk_size: int = 200000, is_iter: bool = False):
        """
           :param env_file: env 文件配置
           :param sql_file: sql 文件配置
           :param chunk_size: 数据抽取的批次大小，默认200000
           :param is_iter: 是否以迭代器模式抽取数据，默认False， 该参数已失效
           :return:
           """
        self._env_file = env_file
        self._sql_file = sql_file

        self._chunk_size = chunk_size
        self._is_iter = is_iter

        # JayDeBeConfig实例对象
        self._jdbc = JayDeBeConfig(env_file, sql_file)

        # SQL配置
        self._sql = self._jdbc.sql_config

    def with_engine(self, db_alias: str = None, ):
        """
        :param db_alias: 数据库标识/别名，从配置文件中获取，区分大小写，如: MAIN、REPLICA、USER
        :param chunk_size: 数据抽取的批次大小，默认200000
        :param is_iter: 是否以迭代器模式抽取数据，默认False， 该参数已失效
        :return:
        """
        # 获取引擎
        self._engine = self._jdbc.manager.get_engine(db_alias)

        # 初始化抽取/提取器对象
        self._dt_extractor = DataExtractor(self._engine, self._jdbc.db_config.FILE_PATH)

        return self

    @property
    def extractor(self):
        return self._dt_extractor.extractor

    @property
    def engine(self):
        return self._engine

    @property
    def to_csv(self):
        return self._data_fetcher()

    @property
    def to_csv_iter(self):
        return self._data_fetcher_iter()

    def _data_fetcher(self):
        @self._dt_extractor.fetch_db_iter(chunk_size=self._chunk_size)
        def fetch_data_func(table_name, start_date=None, end_date=None, T_1=False, fmt="%Y%m%d"):
            """
            :param table_name: Oracle库的表名, 从tables.yml配置文件中获取
            :param T_1: 当天抽取 当天时间-1天的数据
            :param start_date: 数据抽取的开始时间，如 20231001
            :param end_date: 数据抽取的结束时间，如 20240831
            :param fmt: 指定日期过滤格式，默认为%Y%m%d，如：20231001
            :return:
            """

            if T_1:
                # T+1
                start_date = end_date = (
                        datetime.now() - timedelta(days=1, microseconds=0, seconds=0)
                ).strftime(fmt)
            elif start_date is None or end_date is None:
                raise ValueError("start_date or end_date is None")

            print(f"{self._sql['tables'][table_name]['desc']}")

            fetch_sql = self._sql["tables"][table_name]['sql'].format(start_date=start_date, end_date=end_date)

            count_query = re.sub(r"SELECT.+FROM", "SELECT count(1) FROM", fetch_sql, flags=re.DOTALL | re.I)
            count_sql = re.sub(r"ORDER BY.*$", "", count_query, flags=re.DOTALL | re.I)

            print(f"Count SQL({start_date}-{end_date}): ---\n{count_sql.strip()}")
            logger.debug(f"Count SQL({start_date}-{end_date}): ---\n{count_sql.strip()}")

            count = check_data(self._engine, count_sql)[0]

            print(f"Count num: {count}")
            logger.debug(f"Count num: {count}")

            file_date = end_date.replace("-", "")
            file_name = "A_paycenter_{}_D_{}_01".format(table_name.split('.')[1], file_date)

            return [(file_name, file_date, fetch_sql, count)]

        return fetch_data_func

    def _data_fetcher_iter(self):
        @self._dt_extractor.fetch_db_iter(chunk_size=self._chunk_size)
        def fetch_data_iter_func(table_name, start_date, end_date, index=1, mode="month", fmt="%Y%m%d"):
            """
            :param table_name: Oracle库的表名, 从tables.yml配置文件中获取
            :param start_date: 数据抽取的开始时间，如 20231001
            :param end_date: 数据抽取的结束时间，如 20240831
            :param index: 数据抽取的索引，默认1, (可根据日志记录的断点时间，调整index,start_date,end_date从断点开始抽取)
            :param mode: 数据抽取的模式，默认month，可选month, year, day
            :param fmt: 指定日期过滤格式，默认为%Y%m%d，如：20231001
            :return:
            """

            if mode == "day":
                s_e_dates = get_day_dates(start_date, end_date, fmt)
            elif mode == "month":
                # 基线 - 亿级别数据 按月拆
                s_e_dates = get_month_start_end_dates(start_date, end_date, fmt)
            elif mode == "year":
                # 按年拆
                s_e_dates = get_year_start_end_dates(start_date, end_date, fmt)
            else:
                raise Exception("Unknown mode: %s, should be 'month' or 'year'." % mode)

            logger.debug(f'{mode} start - end : {s_e_dates}')

            print(f"{self._sql['tables'][table_name]['desc']}")
            logger.debug(f"{self._sql['tables'][table_name]['desc']}")

            _filter = self._sql["tables"][table_name]['filter']
            if _filter is None:
                raise Exception("No datetime filter.")

            print(f"Datetime during: {start_date} - {end_date}\n-")
            logger.debug(f"Datetime during: {start_date} - {end_date}\n-")

            for _start_date, _end_date in s_e_dates:
                fetch_sql = self._sql["tables"][table_name]['sql'].format(start_date=_start_date, end_date=_end_date)

                # count_sql = f"select count(1) from {table_name} where {_filter} >= '{_start_date}' and {_filter} <='{_end_date}'"
                count_query = re.sub(r"SELECT.+FROM", "SELECT count(1) FROM", fetch_sql, flags=re.DOTALL | re.I)
                count_sql = re.sub(r"ORDER BY.*$", "", count_query, flags=re.DOTALL | re.I)

                print(f"Count SQL({_start_date}-{_end_date}): ---\n{count_sql.strip()}")
                logger.debug(f"Count SQL({_start_date}-{_end_date}): ---\n{count_sql.strip()}")

                count = check_data(self._engine, count_sql)[0]

                print(f"Count num: {count}")
                logger.debug(f"Count num: {count}")
                if count == 0:
                    continue

                file_date = end_date.replace("-", "")
                file_name = "A_paycenter_{}_D_{}_{}".format(table_name.split('.')[1], file_date, "%03d" % index)
                index += 1
                print(f"File: {file_name}")
                logger.debug(f"File: {file_name}")
                yield file_name, end_date, fetch_sql, count

        return fetch_data_iter_func


import jaydebeapi
from jpype import JVMNotFoundException

# 创建连接
class JayDeBeEngine(object):
    def __init__(self, jdbc_driver_name, jdbc_url, jdbc_user, jdbc_password, jdbc_jar):
        """
        Args:
            jdbc_driver_name (str): 驱动名称, 如: com.oceanbase.jdbc.Driver
            jdbc_url (str):  jdbc URL, 如: jdbc:oceanbase://192.168.0.1:2883/TestDB?useUnicode=true&characterEncoding=UTF-8&rewriteBatchedStatements=true
            jdbc_user (str): 数据库用户名, 如: xx@xx#xx
            jdbc_password (str): 数据库密码, 如: 123456
            jdbc_jar (str): JDBC 驱动 JAR 文件路径, 如 /root/oceanbase-client-1.2.3.jar
        """
        self._jdbc_driver_name = jdbc_driver_name
        self._jdbc_url = jdbc_url
        self._jdbc_user = jdbc_user
        self._jdbc_password = jdbc_password
        self._jdbc_jar = jdbc_jar

    def connect(self):
        try:
            conn = jaydebeapi.connect(
                self._jdbc_driver_name, self._jdbc_url, [self._jdbc_user, self._jdbc_password], self._jdbc_jar
                )
        except JVMNotFoundException as e:
            raise JVMNotFoundException(f"{e}\njpype.startJVM启动失败，请检查JAVA_HOME配置. 如:os.environ['JAVA_HOME']='~/anaconda3/envs/dcapp'")
        return conn

    def raw_connection(self):
        return self.connect()

    def __repr__(self):
        return f"<JayDeBeEngine(jdbc_url='{self._jdbc_url}', user='{self._jdbc_user}')>"


#! python3
# -*- encoding: utf-8 -*-

import os
import pytest
from dbgrab.jaydebe import get_engine_manager


def test_engine_map():
    # 使用自定义.env文件
    env_file = os.path.join(os.path.dirname(__file__), '.env.example')

    # 获取引擎管理器
    manager = get_engine_manager(env_file)

    # 列出所有引擎信息
    engines = manager.list_engines()
    print("\n📊 已配置的数据库引擎:")
    for name, info in engines.items():
        print(f"  🔹 {name}: {info}")

    # 获取指定的引擎
    print(f"获取指定的引擎: {manager.get_engine("USER2")}")

    # 获取指定的session
    with manager.get_session("USER2") as session:
        print(f"获取指定的session: {session}")

    # 关闭所有引擎
    manager.close_all_engines()

if __name__ == "__main__":
    pytest.main()


from pydantic import Field
from pydantic_settings import BaseSettings


class DeploymentConfig(BaseSettings):
    """ 应用部署配置信息 """

    CURRENT_VERSION: str = Field(
        description="dbgrab版本信息",
        default="1.1.0",
    )

    DEBUG: bool = Field(
        description="是否开启调试模式",
        default=False,
    )


class FetcherConfig(BaseSettings):
    """ 工作路径基本配置 """
    WORK_PATH: str = Field(..., description="工作路径")
    LOG_PATH: str = Field(..., description="日志路径")
    FILE_PATH: str = Field(..., description="文件保存的路径")


class DeployConfig(
    DeploymentConfig,
    FetcherConfig,
):
    pass


#! python3
# -*- encoding: utf-8 -*-
"""
SQLFetcher SDK - 采集 SQL 数据到pandas csv的工具
"""

from dbgrab.extractor.dt_factory import (
    log,
    logger,
    check_data,
    get_month_start_end_dates,
    get_year_start_end_dates,
    get_day_dates,
)

from dbgrab.extractor.dt_extractor import DataExtractor
from dbgrab.config_generator import ConfigGenerator

from dbgrab.jaydebe import get_engine_manager, set_global_engine_manager
from dbgrab.configs import get_db_config, clear_config_cache

__all__ = [
    "log",
    "logger",
    "check_data",
    "get_month_start_end_dates",
    "get_year_start_end_dates",
    "get_day_dates",
    "DataExtractor",
    "ConfigGenerator",
    "get_engine_manager",
    "set_global_engine_manager",
    "get_db_config",
    "clear_config_cache",
]

__version__ = "0.1.0"



#! python3
# -*- encoding: utf-8 -*-

from typing import Dict, Optional, Any
from rtsf.p_common import FileUtils
from dbgrab import logger
from dbgrab import get_engine_manager


class JayDeBeConfig(object):

    def __init__(self, env_file=".env", sql_file="sql.yml"):
        """
            :param env_file: 环境配置文件路径
            :param sql_file: SQL模板配置文件
            :return:
        """
        self.sql_config: Dict[str, Any] = {}
        self.manager = None
        self.db_config = None

        self._load_sql(sql_file)
        self._init_engine_manager(env_file)

    def _load_sql(self, sql_file):
        """ 初始化SQL模板配置
        Args:
            sql_file (str, optional): SQL模板配置文件路径. 默认值为 'tables.yml'.
        """
        self.sql_config = FileUtils.load_file(sql_file)
        logger.info(f"SQL Tables List: {self.sql_config['tables'].keys()}")

    def _init_engine_manager(self, env_file):
        """ 初始化数据库引擎管理器
        Args:
            env_file (str, optional): 环境配置文件路径.
        """
        # 初始化引擎管理器
        self.manager = get_engine_manager(env_file)
        self.db_config = self.manager._config

        # 打印引擎信息
        engines = self.manager.list_engines()
        logger.info(f"Engines List: {engines.keys()}")





#!/usr/bin/env python3
# -*- encoding: utf-8 -*-


import pytest
from dbgrab import ConfigGenerator
from dbgrab.cons import JayDeBeConfig


env_file="temp/.env"
sql_file="temp/table.yml"


def test_jaydebe_config():
    """测试jaydebe配置加载"""
    print("加载配置文件...")

    generator = ConfigGenerator()
    generator.gen_sql_template(sql_file)
    generator.gen_env_template(env_file)

    j_conf = JayDeBeConfig(env_file=env_file, sql_file=sql_file)
    print(f"SQL配置： {j_conf.sql_config}")
    print(f"引擎管理器： {j_conf.manager}")
    print(f"ENV配置： {[i for i in dir(j_conf.db_config) if i.isupper()]}")

    print("✓ 成功加载数据库配置")



if __name__ == "__main__":
    pytest.main()



#! python3
# -*- encoding: utf-8 -*-

import os


class ConfigGenerator:
    """
    配置文件生成器，基于模板生成database.yml和tables.yml配置文件
    """

    def __init__(self):
        """
        初始化配置生成器
        """
        self.template_dir = os.path.join(os.path.dirname(__file__), 'template')

    def get_template(self, template_name: str, output_path: str = None) -> None:
        """
        获取模板文件并保存到当前目录

        Args:
            template_name: 模板文件名
            output_path: 输出文件路径，如果为None则保存到当前目录
        """
        # 读取模板文件
        template_path = os.path.join(self.template_dir, template_name)
        with open(template_path, 'r', encoding='utf-8') as f:
            template_content = f.read()

        if output_path is None:
            output_path = os.path.join(os.getcwd(), template_name)

        # 写入输出文件
        os.makedirs(os.path.dirname(output_path), exist_ok=True)
        with open(output_path, 'w', encoding='utf-8') as f:
            f.write(template_content)

    def gen_sql_template(self, output_path: str = None) -> None:
        self.get_template('tables.yml', output_path)

    def gen_env_template(self, output_path: str = None) -> None:
        self.get_template('.env.example', output_path)




#! python3
# -*- encoding: utf-8 -*-

import os
from functools import lru_cache
from typing import Dict, Optional
from pydantic import Field
from pydantic_settings import BaseSettings, SettingsConfigDict

from dbgrab.configs.deploy import DeployConfig
from dbgrab.configs.middleware import DynamicDBConfig


@lru_cache()
def get_db_config(env_file: Optional[str] = None) -> DynamicDBConfig:
    """获取数据库配置（带缓存）"""
    if env_file:
        DynamicDBConfig.set_env_file(env_file)

    current_env_file = DynamicDBConfig.get_env_file()

    # 动态创建配置类
    class EnvironmentAwareConfig(
        DeployConfig,
        DynamicDBConfig,
    ):
        model_config = SettingsConfigDict(
            env_file=current_env_file,
            env_file_encoding='utf-8',
            case_sensitive=True,
            extra='ignore',
            env_prefix='DB_ENGINE_',
            env_nested_delimiter='__',
        )
    return EnvironmentAwareConfig()


def clear_config_cache():
    """清除配置缓存"""
    get_db_config.cache_clear()



#! python3
# -*- encoding: utf-8 -*-

import pytest
from dbgrab import log
from dbgrab.dt_fetcher import DataBaseFetcher

env_file= "cmft_conf/.env.cmft"
sql_file= "cmft_conf/tables.yml"
log_file = "temp/mytest.log"

def test_grab_mysql():
    # 设置日志
    log.to_file(log_file)
    print("log: %s" % log_file)

    dbf = DataBaseFetcher(env_file, sql_file).with_engine("CREDIBLE")  # oracle

    # T+1
    print(dbf.to_csv(table_name='credible_db.fetcher_task_record', T_1=True))
    print("✓ 数据抓取器（DataBaseFetcher） T+1抓取功能，测试成功")

    #  指定日期
    print(dbf.to_csv(table_name='credible_db.fetcher_task_record', start_date="20250818", end_date="20251026"))
    print("✓ 数据抓取器（DataBaseFetcher） 指定日期抓取功能，测试成功")

    # 迭代抽取
    print(dbf.to_csv_iter(table_name='credible_db.fetcher_task_record', start_date="20250818", end_date="20251026", mode="day"))
    print("✓ 数据抓取器（DataBaseFetcher） 迭代抽取抓取功能，测试成功")

def test_grab_oceanbase():
    # 设置日志 - oceanbase 库 需要安装java jdk,并配置JAVA_HOME
    log.to_file(log_file)
    print("log: %s" % log_file)

    dbf = DataBaseFetcher(env_file, sql_file).with_engine("TRADE")  # oceanbase

    # T+1
    print(dbf.to_csv(table_name='TRADEDBADM.WTAORDLT', T_1=True))
    print("✓ 数据抓取器（DataBaseFetcher） T+1抓取功能，测试成功")

    #  指定日期
    print(dbf.to_csv(table_name='TRADEDBADM.WTAORDLT', start_date="20251204", end_date="20260204"))
    print("✓ 数据抓取器（DataBaseFetcher） 指定日期抓取功能，测试成功")

    # 迭代抽取
    print(dbf.to_csv_iter(table_name='TRADEDBADM.WTAORDLT', start_date="20200101", end_date="20251026", mode="year"))
    print("✓ 数据抓取器（DataBaseFetcher） 迭代抽取抓取功能，测试成功")


if __name__ == "__main__":
    pytest.main()



#! python3
# -*- encoding: utf-8 -*-

import os
from typing import Dict, Optional
from pydantic import Field
from pydantic_settings import BaseSettings, SettingsConfigDict

from dbgrab.configs.middleware.database_conifg import DatabaseConfig
from dbgrab.configs.middleware.oceanbase_conifg import OceanbaseConfig


class DynamicDBConfig(BaseSettings):
    """动态数据库配置主类"""

    # 数据库配置字典, 传统数据库
    # noinspection PyDataclass
    DATABASES: Dict[str, DatabaseConfig] = Field(default_factory=dict)

    # 数据库配置字典, oceanbase oracle
    # noinspection PyDataclass
    OCEANBASES: Dict[str, OceanbaseConfig] = Field(default_factory=dict)

    model_config = SettingsConfigDict(
        env_file=None,  # 动态设置
        env_file_encoding='utf-8',
        extra='ignore',
        env_prefix='DB_ENGINE_',
        env_nested_delimiter='__',  # 嵌套模型分隔符
    )

    # 动态环境文件路径
    _custom_env_file: Optional[str] = None

    @classmethod
    def set_env_file(cls, env_file: str):
        """设置自定义环境文件"""
        if not os.path.exists(env_file):
            raise FileNotFoundError(f"环境文件不存在: {env_file}")
        cls._custom_env_file = env_file

    @classmethod
    def get_env_file(cls) -> Optional[str]:
        """获取当前环境文件路径"""
        return cls._custom_env_file
